@@ -4,7 +4,6 @@ import json
 import os
 import sys
 import traceback
-
 import six
 
 
@@ -44,11 +43,23 @@ class ADK(object):
                 print("PIPE_INIT_COMPLETE")
                 sys.stdout.flush()
 
+    def apply(self, payload):
+        try:
+            if self.load_result:
+                apply_result = self.apply_func(payload, self.load_result)
+            else:
+                apply_result = self.apply_func(payload)
+            response_obj = self.format_response(apply_result)
+            return response_obj
+        except Exception as e:
+            response_obj = self.create_exception(e)
+            return response_obj
+
     def format_data(self, request):
         if request["content_type"] in ["text", "json"]:
             data = request["data"]
         elif request["content_type"] == "binary":
-            data = self.wrap_binary_data(request["data"])
+            data = self.wrap_binary_data(base64.b64decode(request["data"]))
         else:
             raise Exception("Invalid content_type: {}".format(request["content_type"]))
         return data
@@ -98,9 +109,11 @@ class ADK(object):
             if os.name == "nt":
                 sys.stdin = payload
 
-    def create_exception(self, exception):
+    def create_exception(self, exception, loading_exception=False):
         if hasattr(exception, "error_type"):
             error_type = exception.error_type
+        elif loading_exception:
+            error_type = "LoadingError"
         else:
             error_type = "AlgorithmError"
         response = json.dumps({
@@ -113,33 +126,20 @@ class ADK(object):
         return response
 
     def process_loop(self):
-        response_obj = ""
         for line in sys.stdin:
-            try:
-                request = json.loads(line)
-                formatted_input = self.format_data(request)
-                if self.load_result:
-                    apply_result = self.apply_func(formatted_input, self.load_result)
-                else:
-                    apply_result = self.apply_func(formatted_input)
-                response_obj = self.format_response(apply_result)
-            except Exception as e:
-                response_obj = self.create_exception(e)
-            finally:
-                self.write_to_pipe(response_obj)
+            request = json.loads(line)
+            formatted_input = self.format_data(request)
+            result = self.apply(formatted_input)
+            self.write_to_pipe(result)
 
     def process_local(self, local_payload, pprint):
-        if self.load_result:
-            apply_result = self.apply_func(local_payload, self.load_result)
-        else:
-            apply_result = self.apply_func(local_payload)
-        pprint(self.format_response(apply_result))
-
+        result = self.apply(local_payload)
+        self.write_to_pipe(result, pprint=pprint)
 
     def init(self, local_payload=None, pprint=print):
         self.load()
         if self.loading_exception:
-            load_error = self.create_exception(self.loading_exception)
+            load_error = self.create_exception(self.loading_exception, loading_exception=True)
             self.write_to_pipe(load_error, pprint=pprint)
         elif self.is_local and local_payload:
             self.process_local(local_payload, pprint)
